@@ -1,10 +1,13 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: cmndline.cc,v 1.12 2001/06/09 22:39:48 jgg Exp $
+// $Id: cmndline.cc,v 1.15 2003/02/10 01:40:58 doogie Exp $
 /* ######################################################################
 
    Command Line Class - Sophisticated command line parser
    
+   This source is placed in the Public Domain, do with it what you will
+   It was originally written by Jason Gunthorpe <jgg@debian.org>.
+   
    ##################################################################### */
 									/*}}}*/
 // Include files							/*{{{*/
@@ -17,6 +20,7 @@
 
 #include <apti18n.h>    
 									/*}}}*/
+using namespace std;
 
 // CommandLine::CommandLine - Constructor				/*{{{*/
 // ---------------------------------------------------------------------
@@ -194,13 +198,13 @@ bool CommandLine::HandleOpt(int &I,int argc,const char *argv[],
 	 const char *J;
 	 for (J = Argument; *J != 0 && *J != '='; J++);
 	 if (*J == 0)
-	    return _error->Error(_("Option %s: Configuration item sepecification must have an =<val>."),argv[I]);
+	    return _error->Error(_("Option %s: Configuration item specification must have an =<val>."),argv[I]);
 
 	 // = is trailing
 	 if (J[1] == 0)
 	 {
 	    if (I+1 >= argc)
-	       return _error->Error(_("Option %s: Configuration item sepecification must have an =<val>."),argv[I]);
+	       return _error->Error(_("Option %s: Configuration item specification must have an =<val>."),argv[I]);
 	    Conf->Set(string(Argument,J-Argument),string(argv[I++ +1]));
 	 }
 	 else
