@@ -1,6 +1,6 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
-// $Id: cdrom.cc,v 1.18 2001/05/22 04:01:41 jgg Exp $
+// $Id: cdrom.cc,v 1.21 2004/01/07 20:39:38 mdz Exp $
 /* ######################################################################
 
    CDROM URI method for APT
@@ -18,6 +18,7 @@
 #include <unistd.h>
 
 #include <iostream>
+#include <apti18n.h>
 									/*}}}*/
 
 using namespace std;
@@ -109,7 +110,7 @@ bool CDROMMethod::Fetch(FetchItem *Itm)
       if (FileExists(DFile) == true)
       {
 	 if (ReadConfigFile(Database,DFile) == false)
-	    return _error->Error("Unable to read the cdrom database %s",
+	    return _error->Error(_("Unable to read the cdrom database %s"),
 			  DFile.c_str());
       }
       DatabaseLoaded = true;
@@ -118,15 +119,15 @@ bool CDROMMethod::Fetch(FetchItem *Itm)
    // All non IMS queries for package files fail.
    if (Itm->IndexFile == true || GetID(Get.Host).empty() == true)
    {
-      Fail("Please use apt-cdrom to make this CD recognized by APT."
-	   " apt-get update cannot be used to add new CDs");
+      Fail(_("Please use apt-cdrom to make this CD recognized by APT."
+	   " apt-get update cannot be used to add new CDs"));
       return true;
    }
 
    // We already have a CD inserted, but it is the wrong one
    if (CurrentID.empty() == false && Database.Find("CD::" + CurrentID) != Get.Host)
    {
-      Fail("Wrong CD",true);
+      Fail(_("Wrong CD"),true);
       return true;
    }
    
@@ -137,6 +138,7 @@ bool CDROMMethod::Fetch(FetchItem *Itm)
    while (CurrentID.empty() == true)
    {
       bool Hit = false;
+      Mounted = MountCdrom(CDROM);
       for (unsigned int Version = 2; Version != 0; Version--)
       {
 	 if (IdentCdrom(CDROM,NewID,Version) == false)
@@ -158,24 +160,21 @@ bool CDROMMethod::Fetch(FetchItem *Itm)
 	 
       // I suppose this should prompt somehow?
       if (UnmountCdrom(CDROM) == false)
-	 return _error->Error("Unable to unmount the CD-ROM in %s, it may still be in use.",
+	 return _error->Error(_("Unable to unmount the CD-ROM in %s, it may still be in use."),
 			      CDROM.c_str());
       if (MediaFail(Get.Host,CDROM) == false)
       {
 	 CurrentID = "FAIL";
-	 Fail("Wrong CD",true);
+	 Fail(_("Wrong CD"),true);
 	 return true;
       }
-      
-      MountCdrom(CDROM);
-      Mounted = true;
    }
    
    // Found a CD
    Res.Filename = CDROM + File;
    struct stat Buf;
    if (stat(Res.Filename.c_str(),&Buf) != 0)
-      return _error->Error("File not found");
+      return _error->Error(_("File not found"));
    
    if (NewID.empty() == false)
       CurrentID = NewID;
